import httplib2
from BeautifulSoup import BeautifulSoup
BASE_URL = u"http://onemanga.com"

class OMPage(object):
    def __init__(self,episode,tag):
        self.conn = episode.conn
        self.episode = episode
        self.name = tag.string
        self._url = u"%s%s/" % (episode.url,tag['value'])
        self._img_url = None

    def getImgUrl(self):
        if self._img_url is None:
            page = self.conn.request(self._url,"GET")
            soup = BeautifulSoup(page[1])
            tag = soup.find('img','manga-page')
            self._img_url = tag['src']
        return self._img_url

    def show(self):
        import webbrowser
        webbrowser.open(self.img_url())

    def __repr__(self):
        return u"OMPage(Page %s of %s)" % (self.name,self.episode.name)

    url = property(getImgUrl,None,None,"Image of page.")


class OMEpisode(object):
    def __init__(self,conn,tag):
        self.conn = conn
        self.name = tag.string
        self.url = u"%s%s" % (BASE_URL, tag['href'])
        self._pages = None

    def getPages(self):
        if self._pages is None:
            page = self.conn.request(self.url,"GET")
            soup = BeautifulSoup(page[1])
            select = soup.find('select','page-select')
            self._pages = [ OMPage(self,tag) for tag in select.findChildren() ]
        return self._pages

    def __getitem__(self,i):
        return self.pages[i]

    def __len__(self):
        return len(self.pages)

    def __repr__(self):
        if self._pages is None:
            return u"OMEpisode(%s)" % self.name
        return u"OMEpisode(%s, %s pages)" % (self.name,len(self) or "???")

    pages = property(getPages,None,None,"Pages of manga.")


class OMSeries(object):
    def __init__(self,conn,tag):
        self.conn = conn
        self.name = tag.string
        self.url = u"%s/%s/" % (BASE_URL, tag['value'])
        self._episodes = None

    def getEpisodes(self):
        if self._episodes is None:
            soup = BeautifulSoup(self.conn.request(self.url,"GET")[1])
            tds = soup.findAll('td','ch-subject')
            tags = [ x.findChild() for x in tds ]
            self._episodes = [ OMEpisode(self.conn,tag) for tag in tags ]
            self._episodes.reverse()
        return self._episodes

    def __getitem__(self,i):
        return self.episodes[i]

    def __len__(self):
        return len(self.episodes)

    def __repr__(self):
        if self._episodes is None:
            return u"OMSeries(%s)" % self.name
        return u"OMSeries(%s, %s episodes)" % (self.name,len(self))

    episodes = property(getEpisodes,None,None,"Manga episodes.")


class OMReader(object):
    def __init__(self):
        self.conn = httplib2.Http(".cache")
        self._series = None

    def getSeries(self):
        if self._series is None:
            front = self.conn.request(u"%s/" % BASE_URL, "GET")
            soup = BeautifulSoup(front[1])
            series = soup.findAll('option')
            self._series = [ OMSeries(self.conn,x) for x in soup.findAll('option') if x['value'] != ""]

        return self._series

    def __len__(self):
        return len(self.series)

    def __getitem__(self,i):
        typ = type(i)
        if typ == str or typ == unicode:
            for serie in self.series:
                if serie.name.lower() == i.lower():
                    return serie
        return self.series[i]

    def __repr__(self):
        if self._series is None:
            return u"OMReader(%s/)" % BASE_URL
        return u"OMReader(%s/, %s series)" % (BASE_URL, len(self))

    series = property(getSeries,None,None,"Manga series.")
